@@ -159,7 +159,7 @@ class Interface(object):
         if self.pg3init['secure'] == 1:
             self.sslContext = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
             self.sslContext.check_hostname = False
-        self._mqttc.tls_set_context(self.sslContext)
+            self._mqttc.tls_set_context(self.sslContext)
         self.loop = None
         self.inQueue = queue.Queue()
         self.isyVersion = None
@@ -169,6 +169,7 @@ class Interface(object):
         Interface.__exists = True
         self.custom_params_docs_file_sent = False
         self.custom_params_pending_docs = ''
+        self._levelsList = []
 
         """ persistent data storage for Interface """
         self._ifaceData = Custom(self, 'idata')  # Interface data
@@ -436,10 +437,15 @@ class Interface(object):
             warnings.warn('payload not a dictionary')
             return False
 
+        timeout = 10
         while not self.subscribed:
-            # can we block here?
-            LOGGER.warning('MQTT Send waiting on connection.')
+            if timeout == 0:
+                LOGGER.error('MQTT Send timeout :: {}.'.format(messsage))
+                return
+
+            LOGGER.warning('MQTT Send waiting on connection :: {}'.format(message))
             time.sleep(3)
+            timeout -= 1
             
         try:
             validTypes = ['status', 'command', 'system', 'custom']
@@ -499,6 +505,9 @@ class Interface(object):
                     n.enabled = node['enabled']
                     n.private = node['private']
 
+        if 'logLevelList' in config:
+            self._levelsList = config['logLevelList']
+
         if 'logLevel' in config:
             self.currentLogLevel = config['logLevel'].upper()
             LOGGER.debug('Setting log level to {}'.format(self.currentLogLevel))
@@ -510,7 +519,8 @@ class Interface(object):
                 self.currentLogLevel = 'DEBUG'
 
             GLOBAL_LOGGER.setLevel(self.currentLogLevel)
-            pub.publish(self.LOGLEVEL, None, self.currentLogLevel)
+            level = logging.getLevelName(self.currentLogLevel)
+            pub.publish(self.LOGLEVEL, None, { 'name': self.currentLogLevel, 'level': level }  )
 
         pub.publish(self.CONFIG, None, config)
 
@@ -605,7 +615,7 @@ class Interface(object):
             """
             try:
                 if item.get('key') == 'customparamsdoc':
-                    pubpublish(self.CUSTOMPARAMSDOC, None, item.get('value'))
+                    pub.publish(self.CUSTOMPARAMSDOC, None, item.get('value'))
                 else:
                     value = json.loads(item.get('value'))
 
@@ -681,7 +691,8 @@ class Interface(object):
                 NLOGGER.setLevel(self.currentLogLevel)
                 ILOGGER.setLevel(self.currentLogLevel)
 
-                pub.publish(self.LOGLEVEL, None, item['level'])
+                level = logging.getLevelName(self.currentLogLevel)
+                pub.publish(self.LOGLEVEL, None, { 'name': self.currentLogLevel, 'level': level }  )
 
             except (KeyError, ValueError) as err:
                 LOGGER.error('Failed to set {}: {}'.format(key, err), exc_info=True)
@@ -995,6 +1006,31 @@ class Interface(object):
         LOGGER.debug('Sending message {}'.format(message))
         self.send(message, 'system')
 
+    def addLogLevel(self, name, lvl, str_name):
+        logging.addLevelName(lvl, name)
+
+        # we need the current list
+        lid = 0
+        for l in self._levelsList:
+            lid = int(l['id']) if l['id'] > lid
+
+        # Add new level to list
+        self._levelsList.append( {
+            'id': lid,
+            'name': str_name,
+            'value': name,
+            'level': lvl
+            })
+
+        # send list to PG3
+        message = {'setLogList': {'levels': self._levelsList}}
+        LOGGER.debug('Sending message {}'.format(message))
+        self.send(message, 'system')
+
+    def nodes(self):
+        for n in self.nodes:
+            yield self.nodes[n]
+
     def supports_feature(self, feature):
         LOGGER.warning('The supports_feature() function is deprecated.')
         return True
